"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.opa_set_jwt_secret = exports.opa_check = exports.opa_deinit = exports.opa_init = void 0;
const service_common_1 = require("@crosslab/service-common");
const child_process_1 = require("child_process");
const readline_1 = __importDefault(require("readline"));
const openfga_1 = require("./openfga");
// eslint-disable-next-line @typescript-eslint/no-explicit-any
let opaProcess;
async function opa_init() {
    opaProcess = (0, child_process_1.spawn)('opa', ['run', '-s', '--bundle', 'policy', '--addr', 'localhost:3011'], { stdio: ['ignore', 'pipe', 'pipe'] });
    readline_1.default
        .createInterface({
        input: opaProcess.stderr,
        terminal: false,
    })
        .on('line', line => {
        const json = JSON.parse(line.toString());
        service_common_1.logger.log(json.level, json.message, { ...json, subsystem: 'opa' });
    });
    readline_1.default
        .createInterface({
        input: opaProcess.stdout,
        terminal: false,
    })
        .on('line', line => {
        service_common_1.logger.log('error', line, { subsystem: 'opa' });
    });
    process.on('SIGINT', () => {
        opaProcess.kill('SIGINT');
        process.exit(0);
    });
    process.on('SIGTERM', () => {
        opaProcess.kill('SIGTERM');
        process.exit(0);
    });
    process.on('SIGQUIT', () => {
        opaProcess.kill('SIGQUIT');
        process.exit(0);
    });
    // wait for opa to be ready
    await new Promise(resolve => {
        const interval = setInterval(() => {
            fetch('http://localhost:3011/')
                .then(() => {
                clearInterval(interval);
                resolve();
            })
                .catch(() => {
                // do nothing
            });
        }, 500);
    });
}
exports.opa_init = opa_init;
function opa_deinit() {
    opaProcess.kill(9);
}
exports.opa_deinit = opa_deinit;
async function opa_check(checks) {
    const outputPromise = checks.map((input) => fetch('http://localhost:3011/v1/data/crosslab', {
        method: 'POST',
        body: JSON.stringify({
            input: {
                ...input,
                object: (0, openfga_1.fix_object_without_colon)(input.object),
                ...openfga_1.openfgaOpaData,
            },
        }),
    })
        .then(response => response.json())
        .then(response => {
        service_common_1.logger.info('opa_check', {
            ...input,
            object: (0, openfga_1.fix_object_without_colon)(input.object),
            ...openfga_1.openfgaOpaData,
            response,
        });
        return response;
    })
        .then(json => ({
        result: json.result.allow ?? false,
        reason: json.result.reason ?? 'unknown',
    })));
    return await Promise.all(outputPromise);
}
exports.opa_check = opa_check;
async function opa_set_jwt_secret(secret) {
    await fetch('http://localhost:3011/v1/data/jwt_secret', {
        method: 'PUT',
        body: JSON.stringify(secret),
    });
}
exports.opa_set_jwt_secret = opa_set_jwt_secret;
//# sourceMappingURL=opa.js.map